@@ -1,48 +1,56 @@
-# -*- coding: utf-8 -*-
-
-"""
-Render a FS object as text tree views.
+# coding: utf-8
+"""Render a FS object as text tree views.
 
+Color is supported on UNIX terminals.
 """
 
 from __future__ import print_function
 from __future__ import unicode_literals
 
 import sys
+import typing
 
 from fs.path import abspath, join, normpath
 
+if False:  # typing.TYPE_CHECKING
+    from typing import List, Optional, Text, TextIO, Tuple
+    from .base import FS
+    from .info import Info
 
-def render(fs,
-           path='/',
-           file=None,
-           encoding=None,
-           max_levels=5,
-           with_color=None,
-           dirs_first=True,
-           exclude=None,
-           filter=None):
-    """
-    Render a directory structure in to a pretty tree.
 
-    :param fs: A filesystem.
-    :type fs: A :class:`~fs.base.FS` instance
-    :param file: An open file-like object to render the tree, or
-        ``None`` for stdout.
-    :type file: file or None
-    :type encoding: Unicode encoding, or None to auto-detect.
-    :type encoding: str or None
-    :param int max_levels: Maximum number of levels to display, or None
-        for no maximum.
-    :param bool with_color: Enable terminal color output, or None to
-        auto-detect terminal.
-    :param bool dirs_first: Show directories first.
-    :param list exclude: Option list of directory patterns to exclude
-        from the tree render.
-    :param filter: Optional list of files patterns to match in the tree
-        render.
-    :rtype: tuple
-    :returns: A tuple of ``(<directory count>, <file count>)``.
+def render(fs,                  # type: FS
+           path='/',            # type: Text
+           file=None,           # type: Optional[TextIO]
+           encoding=None,       # type: Optional[Text]
+           max_levels=5,        # type: int
+           with_color=None,     # type: Optional[bool]
+           dirs_first=True,     # type: bool
+           exclude=None,        # type: Optional[List[Text]]
+           filter=None          # type: Optional[List[Text]]
+           ):
+    # type: (...) -> Tuple[int, int]
+    """Render a directory structure in to a pretty tree.
+
+    Arguments:
+        fs (~fs.base.FS): A filesystem instance.
+        path (str): The path of the directory to start rendering
+            from (defaults to root folder, i.e. ``'/'``).
+        file (io.IOBase): An open file-like object to render the
+            tree, or `None` for stdout.
+        encoding (str, optional): Unicode encoding, or `None` to
+            auto-detect.
+        max_levels (int, optional): Maximum number of levels to
+            display, or `None` for no maximum.
+        with_color (bool, optional): Enable terminal color output,
+            or `None` to auto-detect terminal.
+        dirs_first (bool): Show directories first.
+        exclude (list, optional): Option list of directory patterns
+            to exclude from the tree render.
+        filter (list, optional): Optional list of files patterns to
+            match in the tree render.
+
+    Returns:
+        (int, int): A tuple of ``(<directory count>, <file count>)``.
 
     """
     file = file or sys.stdout
@@ -69,29 +77,42 @@ def render(fs,
     line_indent = char_vertline + ' ' * 3
 
     def write(line):
-        """Write a line to the output."""
+        # type: (Text) -> None
+        """Write a line to the output.
+        """
         print(line, file=file)
 
+    # FIXME(@althonos): define functions using `with_color` and
+    #      avoid checking `with_color` at every function call !
+
     def format_prefix(prefix):
-        """Format the prefix lines."""
+        # type: (Text) -> Text
+        """Format the prefix lines.
+        """
         if not with_color:
             return prefix
         return '\x1b[32m%s\x1b[0m' % prefix
 
     def format_dirname(dirname):
-        """Format a directory name."""
+        # type: (Text) -> Text
+        """Format a directory name.
+        """
         if not with_color:
             return dirname
         return '\x1b[1;34m%s\x1b[0m' % dirname
 
     def format_error(msg):
-        """Format an error."""
+        # type: (Text) -> Text
+        """Format an error.
+        """
         if not with_color:
             return msg
         return '\x1b[31m%s\x1b[0m' % msg
 
     def format_filename(fname):
-        """Format a filename."""
+        # type: (Text) -> Text
+        """Format a filename.
+        """
         if not with_color:
             return fname
         if fname.startswith('.'):
@@ -99,17 +120,23 @@ def render(fs,
         return fname
 
     def sort_key_dirs_first(info):
-        """Sort key func with directories first."""
+        # type: (Info) -> Tuple[bool, Text]
+        """Get the info sort function with directories first.
+        """
         return (not info.is_dir, info.name.lower())
 
     def sort_key(info):
-        """Default  key for info."""
+        # type: (Info) -> Text
+        """Get the default info sort function using resource name.
+        """
         return info.name.lower()
 
     counts = {"dirs": 0, "files": 0}
 
     def format_directory(path, levels):
-        """Recursive directory function."""
+        # type: (Text, List[bool]) -> None
+        """Recursive directory function.
+        """
         try:
             directory = sorted(
                 fs.filterdir(path, exclude_dirs=exclude, files=filter),
